@@ -106,7 +106,8 @@ class Route(object):
         :returns: Response if you need to auth the user otherwise None
         """
 
-        if not server_config.getboolean("auth", False):
+        # FIXME: ugly exception to not require authentication for websocket consoles
+        if not server_config.getboolean("auth", False) or request.path.endswith("console/ws"):
             return None
 
         user = server_config.get("user", "").strip()
@@ -199,6 +200,10 @@ class Route(object):
                     response = Response(request=request, route=route)
                     response.set_status(e.status)
                     response.json({"message": e.text, "status": e.status, "path": route, "request": request.json, "method": request.method})
+                except aiohttp.web.HTTPFound as e:
+                    response = Response(request=request, route=route)
+                    response.set_status(e.status)
+                    raise  # must raise to let aiohttp know about the redirection
                 except aiohttp.web.HTTPException as e:
                     response = Response(request=request, route=route)
                     response.set_status(e.status)
@@ -253,10 +258,11 @@ class Route(object):
                 """
                 To avoid strange effect we prevent concurrency
                 between the same instance of the node
-                (excepting when streaming a PCAP file).
+                (excepting when streaming a PCAP file and WebSocket consoles).
                 """
 
-                if "node_id" in request.match_info and not "pcap" in request.path:
+                #FIXME: ugly exceptions for capture and websocket console
+                if "node_id" in request.match_info and not "pcap" in request.path and not request.path.endswith("console/ws"):
                     node_id = request.match_info.get("node_id")
 
                     if "compute" in request.path:
